﻿Imports UrsTinyWebDBServer
''' <summary>
''' Basis für einen Datenbankservice, dessen Daten in einer 
''' <see cref="Dictionary(Of TKey, TValue)"/> gehalten werden und in eine Datei
''' persistiert werden können.
''' </summary>
Public MustInherit Class DictionaryDBBase
   Implements ITinyWebDB

   ''' <summary>
   ''' Ruft den Dateinamen für die Persistierung der Daten ab oder legt ihn fest.
   ''' </summary>
   ''' <returns>Der Dateinamen für die Persistierung der Daten.</returns>
   ''' <remarks>Durch Wechsel des Dateinamens vor einem Aufruf von <see cref="Persist()"/>
   ''' kann ein Generationen-Konzept realisiert werden.</remarks>
   Public Property DBFileName As String

   ''' <summary>
   ''' Gibt an, ob Daten verändert wurden. Wenn <b>True</b> ist eine Persistierung notwendig.
   ''' </summary>
   ''' <returns><b>True</b>, wenn Daten verändert wurden.</returns>
   Public ReadOnly Property IsDirty As Boolean Implements ITinyWebDB.IsDirty

   ''' <summary>
   ''' Die zu Grunde liegende <see cref="Dictionary(Of TKey, TValue)">Dictionary</see>.
   ''' </summary>
   Protected DB As New Dictionary(Of String, String)

   ''' <summary>
   ''' Initalisiert eine neue Instanz der Datenbank.
   ''' </summary>
   ''' <param name="FileName">Der Dateipfad für die persistierten Daten.
   ''' Die Voreinstellung ist "UrsWebBinDB".</param>
   Public Sub New(Optional FileName As String = "UrsWebBinDB")
      DBFileName = FileName
   End Sub

   ''' <summary>
   ''' Liest Daten aus der festgelegten Datei ein.
   ''' </summary>
   ''' <returns><b>True</b>, wenn die Datei vorhanden ist.</returns>
   ''' <remarks>Die Methode ruft in einem SyncKock-Block die Methode 
   ''' <see cref="DoInitialize()"/> der abgeleiten Klasse auf.</remarks>
   Public Function Initialize() As Boolean Implements ITinyWebDB.Initialize
      Dim rtc As Boolean = False
      SyncLock DB
         rtc = DoInitialize()
         _IsDirty = False
      End SyncLock
      Return rtc
   End Function

   ''' <summary>
   ''' Liest Daten aus der festgelegten Datei ein.
   ''' </summary>
   ''' <returns><b>True</b>, wenn die Datei vorhanden ist.</returns>
   ''' <remarks>Siehe <see cref="DictionaryBase.New()"/> und <see cref="DBFileName"/></remarks>
   Protected MustOverride Function DoInitialize() As Boolean

   ''' <summary>
   ''' Serialisiert die im Arbeitsspeicher gehaltene 
   ''' <see cref="Dictionary(Of TKey, TValue)">Dictionary</see> in eine Datei.
   ''' </summary>
   ''' <remarks>Der Dateiname kann über den 
   ''' <see cref="BinaryDictionaryDB.New(String)">Konstruktor</see> oder die Eigenschaft
   ''' <see cref="DBFileName"/> festgelegt werden.<para/>
   ''' Die Methode ruft in einem SyncKock-Block die Methode 
   ''' <see cref="DoPersist()"/> der abgeleiten Klasse auf.</remarks>
   Public Sub Persist() Implements ITinyWebDB.Persist
      SyncLock DB
         DoPersist()
         _IsDirty = False
      End SyncLock
   End Sub

   ''' <summary>
   ''' Serialisiert die im Arbeitsspeicher gehaltene 
   ''' <see cref="Dictionary(Of TKey, TValue)">Dictionary</see> in eine Datei.
   ''' </summary>
   ''' <remarks>Der Dateiname kann über den 
   ''' <see cref="BinaryDictionaryDB.New(String)">Konstruktor</see> oder die Eigenschaft
   ''' <see cref="DBFileName"/> festgelegt werden.</remarks>
   Protected MustOverride Sub DoPersist()

   ''' <summary>Speichert einen Datenwert.</summary>
   ''' <param name="Key">Schlüsselwert, unter dem der Datenwert gespeichert werden soll.</param>
   ''' <param name="Data">Datenwert, der gespeichert werden soll.</param>
   ''' <remarks>Ist unter dem angegebenen Schlüssel bereits ein Wert abgelegt, wird 
   ''' er durch den angegebenen Wert ersetzt.</remarks>
   Public Sub StoreValue(Key As String, Data As String) Implements ITinyWebDB.StoreValue
      SyncLock DB
         If DB.ContainsKey(Key) Then
            DB.Item(Key) = Data
         Else
            DB.Add(Key, Data)
         End If
         _IsDirty = True
      End SyncLock
   End Sub

   ''' <summary>Ruf einen Datenwert aus der Datenbank ab.</summary>
   ''' <param name="Key">Der Schlüssel für den Datenwert.</param>
   ''' <returns>Der zum Schlüsselwert passende Datenwert oder <b>Nothing</b>, 
   ''' wenn unter dem Schlüssel kein Eintrag vorhanden ist.</returns>
   Public Function GetValue(Key As String) As String Implements ITinyWebDB.GetValue
      SyncLock DB
         If DB.ContainsKey(Key) Then
            Return DB.Item(Key)
         Else
            Return Nothing
         End If
      End SyncLock
   End Function

   ''' <summary>Löscht den zum Schlüssel gehörenden Datenbankeintrag.</summary>
   ''' <param name="Key">Schlüsselwert des Datenbankeintrags, der gelöscht werden soll.</param>
   ''' <returns><b>True</b>, wenn der Schlüssel vorhanden war, ansonsten <b>false</b>.</returns>
   Public Function DeleteValue(Key As String) As Boolean Implements ITinyWebDB.DeleteValue
      SyncLock DB
         If DB.ContainsKey(Key) Then
            DB.Remove(Key)
            _IsDirty = True
            Return True
         Else
            Return False
         End If
      End SyncLock
   End Function

   ''' <summary>
   ''' Liefert den gesamten Datenbestand als Json-String.
   ''' </summary>
   ''' <returns>Der gesamte Datenbestand als Json-String.</returns>
   Public Function GetAll() As String Implements ITinyWebDB.GetAll
      Return Json.Net.JsonNet.Serialize(DB)
   End Function
End Class
